import { updateDynamicData, randomNumber, spawnEntityInRegion, updateHerobrineData, herobrineVFX, updateDynamicSettings, updateDynamicEvents, equipRandomItemsToZombie, writeDynamicData } from "./functions.js"
import { world, system, DimensionTypes, EntityComponentTypes } from "@minecraft/server"
import { ActionFormData, ModalFormData } from "@minecraft/server-ui";

import { HerobrineTriggers } from "./scares_triggers.js"
import { totem1, totem2, totem3 } from "./structuresData.js"

console.warn("> Herobrine Lurking Script (HLS) - Initialize!")

export let targetSelected = world?.getAllPlayers()[0];
export let Dimension = world?.getDimension("minecraft:overworld")
export let herobrineData;
export let herobrineSettings;
export let herobrineEvents;

let knockbackTimer = 0;
let attackTimer = 0;

var herobrineDynamic_SETTINGS = {
    nextEvent_timeArray: [80, 200],
    world_vandalism: false,
    herobrine_fog: true,
    special_attacks: false,
    VFX: true
}

var herobrineDynamic_EVENTS = {
    event_ghosting: true,
    event_ghosting_attack: true,
    event_random_event: true,
    event_attack_player: true,
    event_scary_hit: true,
    event_scary_sound: true,
    event_sign_spawn: true,
    event_darken_vision: true,
    event_tunnel_generation: true,
    event_herobrine_chat: false,
    event_own_animals: true,
    event_torch_transform: true,
    event_world_crash: false,
    event_hit_transform: true,
}


var herobrineDynamic_DATA = {
    // Basic Information //
    totem: false,
    spawned: false,
    scareID: "None",

    dayInit: 0,
    nextEvent_time: 0,
    event_time: 0,
    location: null,

    // Scares //
    scares: {
        // Dia 0
        ghosting: {
            id: "event_ghosting",
            day: 0,
            time: [5, 10, 15, 20],
            trigger_isExisted: "ghosting",
            spawnSettings: {
                entityID: "hrb:herobrine",
                securityDistance: 4,
                maxDistance: 14,
                maxHeight: 3,
                effects: [],
                tags: ["ghosting", "statue"],
            }
        },
        scarySound: {
            id: "event_scary_sound",
            day: 0,
            time: [15, 25],
            trigger_spawn: "scary_sound"
        },
        signSpawn: {
            id: "event_sign_spawn",
            day: 0,
            time: [20, 25],
            trigger_spawn: "sign_spawn"
        },

        // Dia 1
        scaryHit: {
            id: "event_scary_hit",
            day: 1,
            time: [60, 80, 100, 120],
            trigger_isExisted: "scaryHit",
            spawnSettings: {
                entityID: "hrb:herobrine",
                securityDistance: 4,
                maxDistance: 10,
                maxHeight: 3,
                effects: [],
                tags: ["scaryHit"],
            }
        },
        randomEvent: {
            id: "event_random_event",
            day: 1,
            time: [10, 15, 20, 25],
            trigger_despawn: "random_event",
            trigger_isExisted: "random_event",
            spawnSettings: {
                entityID: "hrb:herobrine",
                securityDistance: 6,
                maxDistance: 18,
                maxHeight: 3,
                effects: [],
                tags: ["randomEvent", "statue"],
            }
        },
        darkenVision: {
            id: "event_darken_vision",
            day: 1,
            time: [10, 15],
            trigger_spawn: "darken_vision"
        },
        ownAnimals: {
            id: "event_own_animals",
            day: 1,
            time: [30, 40, 60],
            trigger_spawn: "own_animals"
        },
        tunnelGeneration: {
            id: "event_tunnel_generation",
            day: 2,
            time: [10, 15, 20],
            trigger_spawn: "tunnel_generation"
        },

        // Dia 2
        ghostingAttack: {
            id: "event_ghosting_attack",
            day: 2,
            time: [10, 15, 20, 25],
            trigger_isExisted: "ghostingAttack",
            spawnSettings: {
                entityID: "hrb:herobrine",
                securityDistance: 4,
                maxDistance: 14,
                maxHeight: 3,
                effects: [],
                tags: ["ghostingAttack", "statue"],
            }
        },
        herobrineChat: {
            id: "event_herobrine_chat",
            day: 2,
            time: [10, 20, 35, 50],
            trigger_spawn: "herobrine_chat"
        },
        torchTransform: {
            id: "event_torch_transform",
            day: 2,
            time: [10, 15, 20],
            trigger_spawn: "torch_transform"
        },

        // Dia 3
        attackPlayer: {
            id: "event_attack_player",
            day: 3,
            time: [60, 80, 100, 120],
            trigger_isExisted: "attack_player",
            spawnSettings: {
                entityID: "hrb:herobrine",
                securityDistance: 3,
                maxDistance: 8,
                maxHeight: 3,
                effects: [],
                tags: ["attack"],
            }
        },
        world_crash: {
            id: "event_world_crash",
            day: 3,
            time: [20, 25, 40, 50],
            trigger_spawn: "world_crash"
        }
    }
};

/*
world.afterEvents.entityRemove.subscribe(event => {
    const { removedEntityId, typeId } = event
    if (typeId == "hrb:herobrine" && herobrineData.scares[herobrineData.scareID]?.trigger_despawn) {
        // const locationFix = { x: Math.round(herobrineData.location.x), y: Math.round(herobrineData.location.y), z: Math.round(herobrineData.location.z) } Não pode ser número float (Não sei por qual motivo)
        if (removedEntityId.hasTag("random_event")) {
            trigger_randomEvent();
        }
    }
})
*/

world.beforeEvents.worldInitialize.subscribe(initEvent => {

    initEvent.itemComponentRegistry.registerCustomComponent('hrb:script_openSettings', {
        onUse: GUI_herobrineMainMenu
    });

    system.waitTicks(100).then(() => {
        herobrineSelectTarget();
    })
});

world.afterEvents.itemStartUseOn.subscribe(event => {
    if (!herobrineData?.totem) {
        system.runTimeout(() => {
            totemStructure(event)
        }, 1)
    }
})

world.afterEvents.entitySpawn.subscribe(event => {
    const { entity } = event
    if (entity.typeId == "hrb:chicken") {
        entity.addEffect("slow_falling", 9999 * 20, { showParticles: false, amplifier: 5 })
    }
})

world.afterEvents.entityHitEntity.subscribe((event) => {
    const { hitEntity, damagingEntity } = event
    const dimension = world.getDimension(hitEntity.dimension.id)
    if (!damagingEntity) return;

    if (damagingEntity.typeId == "hrb:herobrine" && hitEntity.typeId == "minecraft:player" && damagingEntity.hasTag("scaryHit")) {
        hitEntity.addEffect("blindness", 20 * 5)
        herobrineData.event_time = 0
        updateHerobrineData()
    }

    if (herobrineData?.totem && herobrineEvents?.event_hit_transform) {
        if (damagingEntity.typeId == "minecraft:player") {
            const entityOwnList = {
                "minecraft:pig": "hrb:pig",
                "minecraft:cow": "hrb:cow",
                "minecraft:sheep": "hrb:sheep",
                "minecraft:chicken": "hrb:chicken",
                "minecraft:bee": "hrb:bee"
            };
            const entityTransform = entityOwnList[hitEntity?.typeId]
            if (entityTransform && randomNumber(1, 12) == 1) {
                dimension.getEntitiesAtBlockLocation(hitEntity.location).forEach(entity => {
                    if (entity.id != hitEntity.id) return;
                    targetSelected.playSound("mob.ghast.scream", { pitch: 0.6, volume: 100 })
                    const ownMob = dimension.spawnEntity(entityTransform, hitEntity.location)
                    ownMob.setRotation(entity.getRotation())
                    ownMob.applyImpulse(entity.getVelocity())
                    entity.remove()
                })
            }
        }
    }
})

function GUI_herobrineMainMenu(event) {
    const ui = new ActionFormData()
        .title({ translate: "herobrine.gui_main.title" })
        .body({
            rawtext: [
                { text: "\n" },
                { translate: "herobrine.gui_main.spawned" },
                { text: " " }, { translate: herobrineData?.spawned ? "text.yes" : "text.no" },
                { text: "§r \n \n" },
                { translate: "herobrine.gui_main.actual_event" },
                { text: " " }, { text: `§e${herobrineData?.scareID || "None"}` },
                { text: "\n " }
            ]
        })
        .button({ translate: "herobrine.gui_main.text1" })     // Settings
        .button({ translate: "herobrine.gui_main.text2" })     // Events
        .button({ translate: "herobrine.gui_main.text3" })     // Instant Event
        .button({ translate: herobrineData?.totem ? "herobrine.gui_main.deactivate_totem" : "herobrine.gui_main.activate_totem" }); // Botão dinâmico

    ui.show(event.source).then(response => {
        if (!response.canceled) {
            switch (response.selection) {
                case 0: // Settings
                    GUI_herobrineSettings(event);
                    break;
                case 1: // Events
                    GUI_herobrineEvents(event);
                    break;
                case 2:
                    GUI_herobrineInstantEvent(event);
                    break;
                case 3: // Botão dinâmico (Spawn/Activate)
                    if (herobrineData?.totem) {
                        resetHerobrineInstances();
                        event.source.onScreenDisplay.setActionBar({ translate: "herobrine.item.reset_herobrine.text1" });
                    } else {
                        herobrineData.totem = true;
                        writeDynamicData("herobrineData", herobrineData);
                    }
                    break;
            }
        }
    }).catch(err => {
        console.error("Error showing main menu:", err);
    });
}

function GUI_herobrineInstantEvent(event) {
    if (!herobrineData?.totem) {
        event.source.sendMessage({ translate: "herobrine.warning.totemActivate" });
        return;
    }

    const ui = new ModalFormData()
        .title({ translate: "herobrine.gui_instant.title" });

    const validEvents = [
        { id: "ghosting", translate: "herobrine.gui_instant.ghosting" },
        { id: "ghostingAttack", translate: "herobrine.gui_instant.ghosting_attack" },
        { id: "randomEvent", translate: "herobrine.gui_instant.random_event" },
        { id: "attackPlayer", translate: "herobrine.gui_instant.attack_player" },
        { id: "scaryHit", translate: "herobrine.gui_instant.scary_hit" },
        { id: "scarySound", translate: "herobrine.gui_instant.scary_sound" },
        { id: "signSpawn", translate: "herobrine.gui_instant.sign_spawn" },
        { id: "darkenVision", translate: "herobrine.gui_instant.darken_vision" },
        { id: "tunnelGeneration", translate: "herobrine.gui_instant.tunnel" },
        { id: "ownAnimals", translate: "herobrine.gui_instant.own_animals" },
        { id: "torchTransform", translate: "herobrine.gui_instant.torch_transform" },
        { id: "herobrineChat", translate: "herobrine.gui_instant.chat" }
    ];

    const eventNames = validEvents.map(e => ({ translate: e.translate }));
    const players = world.getAllPlayers();
    const playerNames = players.map(p => p.name);

    ui.dropdown({ rawtext: [{ text: "\n" }, { translate: "herobrine.gui_instant.select_event" }] }, eventNames, 0)
    ui.dropdown({ rawtext: [{ text: "\n" }, { translate: "herobrine.gui_instant.select_player" }] }, playerNames, 0)
    ui.textField({ rawtext: [{ text: "\n" }, { translate: "herobrine.gui_instant.enter_time" }] }, {translate: "herobrine.gui_instant.enter_in_seconds"}, "20")

    ui.show(event.source).then(async response => {
        if (response.canceled) return;

        const selectedEvent = validEvents[response.formValues[0]];
        targetSelected = players[response.formValues[1]];

        // Processa o tempo inserido ou usa tempo aleatório
        let eventTime;
        const inputTime = parseInt(response.formValues[2]);
        if (!isNaN(inputTime) && inputTime > 0) {
            eventTime = inputTime;
        } else {
            // Se o campo estiver vazio ou inválido, usa um tempo aleatório do evento
            const timeArray = herobrineData.scares[selectedEvent.id].time;
            eventTime = timeArray[randomNumber(0, timeArray.length - 1)];
        }

        Dimension = world.getDimension(targetSelected.dimension.id);

        const triggerManager = new HerobrineTriggers(targetSelected, herobrineData, herobrineSettings, Dimension);

        // Limpar instâncias existentes do Herobrine
        const cleanup = () => {
            Dimension.getEntities({ type: "hrb:herobrine" }).forEach(entity => {
                if (herobrineSettings?.VFX) herobrineVFX(entity.location);
                entity.remove();
            });
        };

        cleanup();

        // Atualizar dados do Herobrine
        herobrineData.scareID = selectedEvent.id;
        herobrineData.event_time = eventTime;
        herobrineData.nextEvent_time = 10;
        const data = herobrineData.scares[selectedEvent.id];

        // Spawn do Herobrine se necessário
        if (data?.spawnSettings) {
            const entity = await spawnEntityInRegion(
                data.spawnSettings.entityID,
                targetSelected.location,
                data.spawnSettings.maxDistance,
                data.spawnSettings.maxHeight,
                data.spawnSettings.securityDistance
            );

            if (entity && data.spawnSettings.tags) {
                data.spawnSettings.tags.forEach(tag => entity.addTag(tag));
            }
        }

        // Executar evento
        const eventHandlers = {
            darkenVision: () => triggerManager.darkenVision(),
            herobrineChat: () => triggerManager.herobrineChat(),
            scarySound: () => triggerManager.scarySound(),
            signSpawn: () => triggerManager.signSpawn(),
            ownAnimals: () => triggerManager.ownAnimals(),
            torchTransform: () => triggerManager.torchTransform(),
            tunnelGeneration: () => triggerManager.tunnelGeneration(),
            randomEvent: () => triggerManager.randomEvent()
        };

        const handler = eventHandlers[selectedEvent.id];
        if (handler) handler();

        updateHerobrineData();
        event.source.sendMessage({ translate: "herobrine.gui_instant.triggered" });
    }).catch(err => {
        console.error("Error in instant event UI:", err);
    });
}

function GUI_herobrineSettings(event) {
    if (!herobrineData?.totem) {
        event.source.sendMessage({ translate: "herobrine.warning.totemActivate" });
        return;
    }

    const ui = new ModalFormData()
        .title({ translate: "herobrine.gui_settings.title" })
        .textField({ translate: "herobrine.gui_settings.text1" }, `${herobrineSettings.nextEvent_timeArray[0]}-${herobrineSettings.nextEvent_timeArray[1]}`, `${herobrineSettings.nextEvent_timeArray[0]}-${herobrineSettings.nextEvent_timeArray[1]}`)
        .toggle({ translate: "herobrine.gui_settings.text2" }, herobrineSettings.world_vandalism)
        .toggle({ translate: "herobrine.gui_settings.text3" }, herobrineSettings.VFX)
        .toggle({ translate: "herobrine.gui_settings.text4" }, herobrineSettings.herobrine_fog);

    ui.show(event.source).then(data => {
        if (!data.canceled) {
            // Processa o campo de texto que contém o tempo para invocar o Herobrine
            const timeInput = data.formValues[0];
            const timeParts = timeInput.split('-');

            if (timeParts.length === 2) {
                let startTime = parseInt(timeParts[0].trim(), 10);
                let endTime = parseInt(timeParts[1].trim(), 10);

                if (!isNaN(startTime) && !isNaN(endTime)) {
                    // Se os tempos estiverem invertidos, troca os valores
                    if (startTime > endTime) {
                        const temp = startTime;
                        startTime = endTime;
                        endTime = temp;
                    } else if (startTime === endTime) {
                        endTime = startTime + 1;
                    }
                    herobrineSettings.nextEvent_timeArray = [startTime, endTime];
                    console.warn(`Tempo de invocação atualizado para: ${startTime}-${endTime}`);
                } else {
                    console.warn("Formato de tempo inválido. Os valores precisam ser numéricos.");
                }
            } else {
                event.source.sendMessage({ translate: "herobrine.gui_settings.warning1" });
            }

            // Processa os valores dos toggles (que estão nas posições 1, 2 e 3)
            const toggleKeys = [
                "world_vandalism",
                "VFX",
                "herobrine_fog"
            ];

            toggleKeys.forEach((key, index) => {
                // index + 1 para pular o campo de texto
                const newValue = data.formValues[index + 1];
                if (newValue !== herobrineSettings[key]) {
                    herobrineSettings[key] = newValue;
                    console.warn(`${key} configurado para ${newValue}`);
                }
            });

            // Atualiza os dados dinamicamente
            updateDynamicSettings();
        }
    }).catch(err => {
        console.error(`Erro ao exibir a interface de configurações do Herobrine: ${err}`);
    });
}


function GUI_herobrineEvents(event) {
    if (!herobrineData?.totem) {
        event.source.sendMessage({ translate: "herobrine.warning.totemActivate" });
        return;
    }

    const ui = new ModalFormData()
        .title({ translate: "herobrine.gui_events.title" })
        .toggle({ translate: "herobrine.gui_events.text1" }, herobrineEvents.event_ghosting)
        .toggle({ translate: "herobrine.gui_events.text2" }, herobrineEvents.event_ghosting_attack)
        .toggle({ translate: "herobrine.gui_events.text3" }, herobrineEvents.event_random_event)
        .toggle({ translate: "herobrine.gui_events.text4" }, herobrineEvents.event_attack_player)
        .toggle({ translate: "herobrine.gui_events.text5" }, herobrineEvents.event_scary_hit)
        .toggle({ translate: "herobrine.gui_events.text6" }, herobrineEvents.event_scary_sound)
        .toggle({ translate: "herobrine.gui_events.text7" }, herobrineEvents.event_herobrine_chat)
        .toggle({ translate: "herobrine.gui_events.text8" }, herobrineEvents.event_sign_spawn)
        .toggle({ translate: "herobrine.gui_events.text9" }, herobrineEvents.event_darken_vision)
        .toggle({ translate: "herobrine.gui_events.text10" }, herobrineEvents.event_own_animals)
        .toggle({ translate: "herobrine.gui_events.text11" }, herobrineEvents.event_torch_transform)
        .toggle({ translate: "herobrine.gui_events.text12" }, herobrineEvents.event_tunnel_generation)
        .toggle({ translate: "herobrine.gui_events.text13" }, herobrineEvents.event_world_crash)
        .toggle({ translate: "herobrine.gui_events.text14" }, herobrineEvents.event_hit_transform)


    ui.show(event.source).then(data => {
        if (!data.canceled) {
            const toggleKeys = [
                "event_ghosting",
                "event_ghosting_attack",
                "event_random_event",
                "event_attack_player",
                "event_scary_hit",
                "event_scary_sound",
                "event_herobrine_chat",
                "event_sign_spawn",
                "event_darken_vision",
                "event_own_animals",
                "event_torch_transform",
                "event_tunnel_generation",
                "event_world_crash",
                "event_hit_transform"
            ];

            const activeEvents = data.formValues.filter(value => value === true);

            if (activeEvents.length === 0) {
                herobrineEvents[toggleKeys[0]] = true;
            } else {
                toggleKeys.forEach((key, index) => {
                    herobrineEvents[key] = data.formValues[index];
                });
            }

            console.warn("Updated Herobrine events:", herobrineEvents);

            updateDynamicEvents();

            event.source.sendMessage({ translate: "herobrine.gui_events.success" });
        }
    }).catch(err => {
        console.error(`Error displaying Herobrine settings UI: ${err}`);
        event.source.sendMessage({ translate: "herobrine.gui_events.error" });
    });
}

async function selectTriggerAndScare() {
    herobrineSelectTarget();

    const triggerManager = new HerobrineTriggers(targetSelected, herobrineData, herobrineSettings, Dimension);
    const currentDay = world.getDay();

    // Filtra sustos válidos para o dia atual
    const validScares = Object.entries(herobrineData.scares)
        .filter(([, scare]) => {
            const scareDay = parseInt(herobrineData.dayInit) + parseInt(scare.day);
            return scareDay <= currentDay && herobrineEvents[scare.id];
        })
        .map(([id]) => id);

    if (validScares.length === 0) {
        console.warn("Nenhum susto válido encontrado para o dia atual.");
        updateHerobrineData();
        return;
    }

    // Seleciona susto aleatório
    herobrineData.scareID = validScares[randomNumber(0, validScares.length - 1)];
    const data = herobrineData.scares[herobrineData.scareID];

    // Define próximo horário de evento
    herobrineData.nextEvent_time = randomNumber(herobrineSettings.nextEvent_timeArray[0], herobrineSettings.nextEvent_timeArray[1])

    // Helper para spawn de entidades com retry
    async function spawnEntity(spawnData, location) {
        const maxAttempts = 4;

        for (let attempts = 1; attempts <= maxAttempts; attempts++) {
            try {
                const entitySpawned = await spawnEntityInRegion(
                    spawnData.spawnSettings.entityID,
                    location,
                    spawnData.spawnSettings.maxDistance,
                    spawnData.spawnSettings.maxHeight,
                    spawnData.spawnSettings.securityDistance
                );

                if (entitySpawned) {
                    if (spawnData.spawnSettings.tags?.length) {
                        spawnData.spawnSettings.tags.forEach(tag => entitySpawned.addTag(tag));
                    }
                    console.log(`Entidade ${spawnData.spawnSettings.entityID} spawnada na tentativa ${attempts}`);
                    return entitySpawned;
                }
            } catch (error) {
                console.error(`Tentativa ${attempts} falhou:`, error);
            }
        }

        console.error(`Limite de ${maxAttempts} tentativas atingido para ${spawnData.spawnSettings.entityID}`);
        return null;
    }

    // Spawn do Herobrine
    let herobrineSpawned = null;
    if (data?.spawnSettings) {
        herobrineSpawned = await spawnEntity(data, targetSelected.location);
    }

    // Efeitos aleatórios após spawn
    if (herobrineSpawned && randomNumber(1, 5) === 1) {
        triggerManager.scarySound();
    }

    // Mapeamento de triggers para métodos da classe
    const triggers = {
        darken_vision: () => triggerManager.darkenVision(),
        herobrine_chat: () => triggerManager.herobrineChat(),
        scary_sound: () => triggerManager.scarySound(),
        sign_spawn: (blockID, loc = targetSelected.location, rad, height, minDistance) => triggerManager.signSpawn(blockID, loc, rad, height, minDistance),
        own_animals: () => triggerManager.ownAnimals(),
        torch_transform: () => triggerManager.torchTransform(),
        tunnel_generation: () => triggerManager.tunnelGeneration(),
        world_crash: () => { if (randomNumber(1, 5) === 1) triggerManager.herobrineCrash(); }
    };

    // Executa o trigger correspondente
    const triggerFunction = triggers[data?.trigger_spawn];

    if (triggerFunction) triggerFunction();

    // Atualiza dados do Herobrine
    herobrineData.event_time = data.time[randomNumber(0, data.time.length - 1)];
    updateHerobrineData();
}

function resetHerobrineInstances() {
    DimensionTypes.getAll().forEach(dimensionList => {
        world.getDimension(dimensionList.typeId).getEntities({ type: "hrb:herobrine" }).forEach(hb => {
            if (herobrineSettings?.VFX) herobrineVFX(hb.location);
            hb.remove();
        })
    })
    herobrineData = null;
    world.setDynamicProperty("herobrineSettings", null)
    world.setDynamicProperty("herobrineEvents", null)
    world.setDynamicProperty("herobrineData", null)
}

function totemStructure(event) {
    const block = event.block
    const dimensionLocal = world.getDimension(event.source.dimension.id)
    const locationStandard = { x: block.location.x, y: block.location.y, z: block.location.z }
    const fireLoc = { x: locationStandard.x, y: locationStandard.y + 1, z: locationStandard.z }
    const fireBlock = dimensionLocal.getBlock(fireLoc).typeId

    function structuresIsValid(structureFormat) {

        let blocksCorrect = 0

        structureFormat.forEach(obj => {
            const locatioFormat = { x: obj.x, y: obj.y, z: obj.z }
            const blockInLoc = dimensionLocal.getBlock(locatioFormat)
            if (blockInLoc.typeId == obj.blockID) {
                blocksCorrect++
            }
        })

        if (blocksCorrect == structureFormat.length) {
            return true
        } else {
            return false
        }
    };

    const isValid_1 = structuresIsValid(totem1(locationStandard), event.player)
    const isValid_2 = structuresIsValid(totem2(locationStandard), event.player)
    const isValid_3 = structuresIsValid(totem3(locationStandard), event.player)

    if (isValid_1 || isValid_2 || isValid_3 && !herobrineData.totem) {
        if (fireBlock == "minecraft:fire") {

            const lightning_boltLoc = { x: block.location.x, y: block.location.y + 1, z: block.location.z };
            const centerBlock = { x: block.location.x + 0.5, y: block.location.y + 1, z: block.location.z + 0.5 };

            dimensionLocal.spawnEntity("hrb:herobrine", centerBlock).addTag("statue")

            dimensionLocal.spawnEntity("minecraft:lightning_bolt", lightning_boltLoc);

            world.sendMessage({ translate: "herobrine.worldJoin.message" });

            herobrineData.totem = true;
            herobrineData.event_time = 2;
            herobrineData.nextEvent_time = randomNumber(30, 60);
            herobrineData.dayInit = world.getDay();

            updateHerobrineData();
        }
    }
}

function herobrineSelectTarget() {
    const players = world.getAllPlayers();
    targetSelected = players.length > 1 ? players[randomNumber(0, players.length - 1)] : players[0];
    Dimension = world.getDimension(targetSelected.dimension.id);
}

function devMode() {
    try {
        world.getPlayers().forEach(player => {
            if (player.hasTag("debug")) {
                player.onScreenDisplay.setActionBar(`Debug Mode\n\nNext Event: ${herobrineData?.nextEvent_time}\nTime: ${herobrineData?.event_time}\nDay Totem: ${herobrineData?.dayInit}\nHerobrine AP: ${herobrineData.scareID}\nTotem: ${herobrineData?.totem}\nActive: ${herobrineData?.spawned}\nLocation: ${JSON.stringify(herobrineData?.location)}`)
            }
        })
    } catch { }
}

function herobrineUpdateTicks() {
    try {
        const triggerManager = new HerobrineTriggers(targetSelected, herobrineData, herobrineSettings, Dimension);
        const herobrineEntities = Dimension.getEntities({ type: "hrb:herobrine" });
        const mainHerobrine = herobrineEntities[0];
        const health = mainHerobrine?.getComponent("minecraft:health");

        // --- Gerenciamento do timer de ataque ---
        attackTimer++;

        if (health?.currentValue <= 500 && attackTimer >= 20) {
            mainHerobrine.runCommand("replaceitem entity @s slot.weapon.mainhand 0 diamond_sword");
            if (randomNumber(1, 20) === 1) {
                const zombieCount = randomNumber(6, 12);
                for (let i = 0; i < zombieCount; i++) {
                    spawnEntityInRegion("minecraft:zombie", mainHerobrine.location, 3, 2, 1).then(zombie => equipRandomItemsToZombie(zombie));
                    if (herobrineSettings.world_vandalism) {
                        spawnEntityInRegion("minecraft:silverfish", mainHerobrine.location, 3, 2, 1);
                    }
                }
            }
            if (randomNumber(1, 20) === 1 && herobrineSettings.world_vandalism) {
                const lightningCount = randomNumber(8, 16);
                for (let i = 0; i < lightningCount; i++) {
                    spawnEntityInRegion("minecraft:lightning_bolt", mainHerobrine.location, 3, 2, 1);
                }
            };
            attackTimer = 0;
        }

        // --- Função para remover todas as entidades Herobrine ---
        const removeHerobrineEntities = () => {
            herobrineEntities.forEach(entity => {
                if (entity.hasTag("randomEvent")) {
                    triggerManager.randomEvent();
                }
                if (herobrineSettings.VFX) {
                    herobrineVFX(entity.location);
                }
                attackTimer = 0;
                entity.remove();
            });
        };

        if (herobrineData?.event_time === 0 && herobrineData?.totem) {
            removeHerobrineEntities();
            if (herobrineData?.nextEvent_time === 0) {
                selectTriggerAndScare();
            }
        }

        // --- Garante que exista apenas uma entidade Herobrine ---
        if (herobrineEntities.length > 1) {
            herobrineEntities[1].remove();
        }

        for (const entity of Dimension.getEntities()) {
            const rideableComponent = entity.getComponent("minecraft:rideable");
            if (rideableComponent) {
                rideableComponent.getRiders()
                    .filter(rider => rider.typeId === "hrb:herobrine")
                    .forEach(rider => rideableComponent.ejectRider(rider));
            }
        }

        // --- Processamento quando há apenas uma entidade Herobrine e o totem está ativo ---
        if (herobrineEntities.length === 1 && herobrineData?.totem) {
            herobrineEntities.forEach(herobrine => {
                // Aplicar knockback a jogadores próximos
                const nearbyPlayers = Dimension.getPlayers({ location: herobrine.location, maxDistance: 0.5 });
                nearbyPlayers.forEach(player => {
                    knockbackTimer++;
                    if (knockbackTimer >= 10) {
                        const { x: dx, z: dz } = herobrine.getViewDirection();
                        player.applyKnockback(dx, dz, randomNumber(-0.3, 0.3), randomNumber(-0.3, 0.3));
                        player.applyDamage(1);
                        knockbackTimer = 0;
                    }
                });

                // Aplicar efeitos se for uma estátua
                if (herobrine.getTags().includes("statue")) {
                    herobrine.addEffect("minecraft:slowness", 1200, { showParticles: false, amplifier: 255 });
                    herobrine.addEffect("minecraft:weakness", 1200, { showParticles: false, amplifier: 255 });
                }

                // --- Gerenciamento dos sustos ativos ---
                const currentScare = herobrineData.scares[herobrineData.scareID];

                const processHerobrineInView = (herobrineInView, player) => {
                    const viewAction = () => {
                        if (herobrineSettings.VFX) {
                            herobrineVFX(herobrineInView.entity.location);
                        }
                        herobrineInView.entity.remove();
                        herobrineData.event_time = 0;
                        updateHerobrineData();
                    };

                    switch (currentScare?.id) {
                        case "event_random_event":
                            triggerManager.randomEvent();
                            viewAction();
                            if (randomNumber(1, 3) === 1) {
                                player.addEffect("blindness", 200); // Duração: 20*10
                                triggerManager.scarySound();
                            }
                            break;
                        case "event_ghosting":
                            viewAction();
                            if (randomNumber(1, 8) === 1) {
                                player.addEffect("blindness", 200);
                            }
                            break;
                        case "event_ghosting_attack":
                            if (mainHerobrine.hasTag("ghostingAttack")) {
                                player.addEffect("blindness", 200);
                                const playerViewDir = player.getViewDirection();
                                herobrineInView.entity.teleport({
                                    x: player.location.x + playerViewDir.x * 0.8,
                                    y: player.location.y + playerViewDir.y * 0.8,
                                    z: player.location.z + playerViewDir.z * 0.8,
                                }, { facingLocation: player.location });
                                herobrineInView.entity.addTag("attack");
                                herobrineInView.entity.removeTag("statue");
                                herobrineInView.entity.removeTag("ghostingAttack");
                                herobrineInView.entity.removeEffect("slowness");
                                herobrineInView.entity.removeEffect("weakness");
                                herobrineData.event_time = randomNumber(40, 60);
                                updateHerobrineData();
                            }
                            break;
                        default:
                            break;
                    }
                };

                // Processar entidades Herobrine na direção da visão de cada jogador
                world.getAllPlayers().forEach(player => {
                    const entitiesInView = player.getEntitiesFromViewDirection({ type: "hrb:herobrine", includeLiquidBlocks: true });
                    entitiesInView?.forEach(herobrineInView => {
                        processHerobrineInView(herobrineInView, player);
                    });
                });
            });

            // Atualiza a localização do Herobrine com precisão de 2 casas decimais
            herobrineData.location = {
                x: parseFloat(mainHerobrine.location.x.toFixed(2)),
                y: parseFloat(mainHerobrine.location.y.toFixed(2)),
                z: parseFloat(mainHerobrine.location.z.toFixed(2)),
            };
            herobrineData.spawned = true;
        } else {
            herobrineData.spawned = false;
        }

        // Atualizar dados dinâmicos
        updateHerobrineData();
    } catch (error) {
        // console.warn(error)
    }
}

system.runInterval(() => {

    // Atualiza os Dynamics do Mundo
    herobrineSettings = updateDynamicData("herobrineSettings", herobrineDynamic_SETTINGS)
    herobrineEvents = updateDynamicData("herobrineEvents", herobrineDynamic_EVENTS)
    herobrineData = updateDynamicData("herobrineData", herobrineDynamic_DATA)

    world.getAllPlayers().forEach(player => {
        herobrineUpdateTicks();
        if (herobrineData?.totem) {
            if (herobrineSettings?.herobrine_fog) Dimension.runCommandAsync(`fog "${player.name}" push hrb:fog_herobrine herobrineFog`);
            if (!herobrineSettings?.herobrine_fog) Dimension.runCommandAsync(`fog "${player.name}" remove herobrineFog`);
        } else {
            Dimension.runCommandAsync(`fog "${player.name}" remove herobrineFog`);
        }
    })
    devMode();
}, 2.5)

system.runInterval(() => {
    if (herobrineData?.totem) {
        if (herobrineData.event_time == 0 && herobrineData.nextEvent_time > 0) {
            herobrineData.nextEvent_time--
        }
        if (herobrineData.event_time > 0) {
            herobrineData.event_time--
        }
        updateHerobrineData()
    }
}, 20);