import { randomNumber, spawnEntityInRegion, BlockManager, calculateDistance, equipRandomItemsToZombie, BlockChecker } from "./functions.js";
import { world, system } from "@minecraft/server";

export class HerobrineTriggers {
    constructor(targetSelected, herobrineData, herobrineSettings, Dimension) {
        this.targetSelected = targetSelected;
        this.herobrineData = herobrineData;
        this.herobrineSettings = herobrineSettings;
        this.Dimension = Dimension;
    }

    darkenVision() {
        this.targetSelected.camera.fade({
            fadeTime: { fadeInTime: 1, holdTime: 3, fadeOutTime: 1 },
            fadeColor: { blue: 0, green: 0, red: 0 }
        });

        this.targetSelected.onScreenDisplay.setTitle(
            { text: " " },
            {
                fadeInDuration: 20,
                fadeOutDuration: 20,
                stayDuration: 50,
                subtitle: {
                    rawtext: [
                        { text: "§c" },
                        { translate: `herobrine.message.scared_${randomNumber(1, 16)}` }
                    ]
                }
            }
        );
    }

    herobrineCrash() {
        this.herobrineCrash()
    }

    ownAnimals() {
        const entityOwnList = {
            "minecraft:pig": "hrb:pig",
            "minecraft:cow": "hrb:cow",
            "minecraft:sheep": "hrb:sheep",
            "minecraft:chicken": "hrb:chicken",
            "minecraft:bee": "hrb:bee"
        };

        const entitiesInPlayerLoc = this.Dimension.getEntities({
            location: this.targetSelected.location,
            maxDistance: randomNumber(30, 45)
        });

        entitiesInPlayerLoc.forEach(entityInRegion => {
            const transformEntityID = entityOwnList[entityInRegion.typeId];
            if (transformEntityID) {
                try {
                    const ownSpawn = this.Dimension.spawnEntity(transformEntityID, entityInRegion.location);
                    ownSpawn.setRotation(entityInRegion.getRotation());

                    const isBaby = entityInRegion.hasComponent("minecraft:is_baby");
                    if (isBaby) {
                        ownSpawn.triggerEvent("minecraft:entity_born");
                    }

                    entityInRegion.remove();
                } catch (e) {
                    console.warn(`Failed to transform entity ${entityInRegion.typeId} at ${JSON.stringify(entityInRegion.location)}: ${e}`);
                }
            }
        });
    }

    torchTransform() {
        const blockManager = new BlockManager();
        let transformed = false;

        if (this.targetSelected.location.y > 45 && randomNumber(1, 4) === 1) {
            blockManager.squareGetBlocks({
                locationStart: this.targetSelected.location,
                allSides: false,
                blockTargetID: "minecraft:torch"
            });
            transformed = true;
        } else if (this.targetSelected.location.y <= 45 && randomNumber(1, 2) == 1) {
            blockManager.squareGetBlocks({
                locationStart: this.targetSelected.location,
                allSides: false,
                blockTargetID: "minecraft:torch"
            });
            transformed = true;
            if (randomNumber(1, 2) == 1) {
                this.scarySound();
            }
        }
    }

    tunnelGeneration() {
        if (randomNumber(1, 5) == 1) {
            const minYCoordinate = -60;

            const areBothBlocksAir = (position) => {
                try {
                    const lowerBlock = this.Dimension.getBlock(position);
                    const upperBlock = this.Dimension.getBlock({
                        x: position.x,
                        y: position.y + 1,
                        z: position.z
                    });
                    return (lowerBlock?.typeId === "minecraft:air" && !lowerBlock?.isLiquid) && (upperBlock?.typeId === "minecraft:air" && !upperBlock?.isLiquid);
                } catch { return false; }
            };

            const findObstaclePosition = (origin, direction, range = 60) => {
                for (let i = 0; i < range; i++) {
                    const pos = {
                        x: origin.x + direction.x * i,
                        y: origin.y,
                        z: origin.z + direction.z * i
                    };
                    if (pos.y < minYCoordinate || pos.y > 319) continue;
                    if (!areBothBlocksAir(pos)) {
                        return pos;
                    }
                }
                return null;
            };

            const generateTunnelPath = (origin, direction, range = 24) => {
                const path = [];
                for (let i = 0; i < range; i++) {
                    const basePos = {
                        x: origin.x + direction.x * i,
                        y: origin.y,
                        z: origin.z + direction.z * i
                    };
                    if (basePos.y < minYCoordinate || basePos.y + 1 > 319) continue;
                    path.push(basePos);
                    path.push({
                        x: basePos.x,
                        y: basePos.y + 1,
                        z: basePos.z
                    });
                }
                return path;
            };

            const hasSolidBlockInDirection = (pos, faceDirection) => {
                const offset = { x: 0, y: 0, z: 0 };
                switch (faceDirection) {
                    case "north": offset.z = -1; break;
                    case "south": offset.z = 1; break;
                    case "east": offset.x = 1; break;
                    case "west": offset.x = -1; break;
                    default: return false;
                }
                const adjacentPos = {
                    x: pos.x + offset.x,
                    y: pos.y,
                    z: pos.z + offset.z
                };

                try {
                    const adjacentBlock = this.Dimension.getBlock(adjacentPos);
                    return !!adjacentBlock &&
                        adjacentBlock.typeId !== "minecraft:air" &&
                        !adjacentBlock.isLiquid;
                } catch {
                    return false;
                }
            };

            const directions = [
                { x: 1, y: 0, z: 0 },
                { x: -1, y: 0, z: 0 },
                { x: 0, y: 0, z: 1 },
                { x: 0, y: 0, z: -1 }
            ];
            const selectedDirection = directions[randomNumber(0, directions.length - 1)];

            let startY = Math.round(this.targetSelected.location.y);
            startY = Math.max(startY, minYCoordinate);
            startY = Math.min(startY, 318);

            const playerFloorPos = { x: Math.round(this.targetSelected.location.x), y: startY, z: Math.round(this.targetSelected.location.z) };

            const obstaclePos = findObstaclePosition(playerFloorPos, selectedDirection, 50);
            const startPos = obstaclePos ? obstaclePos : playerFloorPos;


            const tunnelPath = generateTunnelPath(startPos, selectedDirection, 50);
            tunnelPath.forEach(pos => {
                try {
                    this.Dimension.setBlockType(pos, "minecraft:air");
                } catch (e) {
                }
            });

            const torchPositions = [];
            for (let i = 0; i < 50; i++) {
                const torchY = startPos.y + 1;
                if (torchY >= minYCoordinate && torchY <= 319) {
                    torchPositions.push({
                        x: startPos.x + selectedDirection.x * i,
                        y: torchY,
                        z: startPos.z + selectedDirection.z * i
                    });
                }
            }


            torchPositions.forEach((pos, index) => {
                if (index % 4 === 0) {
                    let torchDirection = null;
                    if (selectedDirection.x !== 0) {
                        if (hasSolidBlockInDirection(pos, "north")) torchDirection = "north";
                        else if (hasSolidBlockInDirection(pos, "south")) torchDirection = "south";
                    } else if (selectedDirection.z !== 0) {
                        if (hasSolidBlockInDirection(pos, "east")) torchDirection = "east";
                        else if (hasSolidBlockInDirection(pos, "west")) torchDirection = "west";
                    }

                    if (torchDirection) {
                        try {
                            this.Dimension.setBlockType(pos, "minecraft:redstone_torch");
                            const torchBlock = this.Dimension.getBlock(pos);
                            if (torchBlock && torchBlock.permutation) {
                                const newState = torchBlock.permutation.withState("torch_facing_direction", torchDirection);
                                torchBlock.setPermutation(newState);
                            }
                        } catch (e) {
                        }
                    }
                }
            });
        }
    }

    scarySound() {
        const soundsInMiner = [
            "scary.dirt_breaking.sound",
            "scary.stone_breaking.sound",
        ];
        const soundsDefault = [
            "ambient.basalt_deltas.mood",
            "ambient.cave",
            "ambient.crimson_forest.mood",
            "ambient.warped_forest.mood"
        ];

        try {
            const playerY = this.targetSelected.location.y;
            let soundToPlay = soundsDefault[randomNumber(0, soundsDefault.length - 1)];

            if (playerY < 40) {
                if (randomNumber(1, 20) == 1) {
                    soundToPlay = soundsInMiner[1];
                }
            } else if (playerY >= 40 && playerY < 90) {
                if (randomNumber(1, 10) == 1) {
                    soundToPlay = soundsInMiner[0];
                }
            }

            this.targetSelected.playSound(soundToPlay);
        } catch (e) {
            console.warn(`Error playing scary sound: ${e}`);
        }
    }

    signSpawn(blockID = "minecraft:standing_sign", loc = this.targetSelected.location, rad = 8, height = 2, minDistance = 4) {
        try {
            const originX = Math.round(loc.x);
            const originY = Math.round(loc.y);
            const originZ = Math.round(loc.z);

            const generateCoordinatesWithinVolume = () => {
                const validPositions = [];
                const worldMinY = -64;
                const worldMaxY = 319;
                const minY = Math.max(worldMinY, originY - height);
                const maxY = Math.min(worldMaxY, originY + height);

                for (let dx = -rad; dx <= rad; dx++) {
                    for (let dz = -rad; dz <= rad; dz++) {
                        const distance = Math.sqrt(dx * dx + dz * dz);
                        if (distance < minDistance || distance > rad) {
                            continue;
                        }

                        for (let currentY = minY; currentY <= maxY; currentY++) {
                            const spX = originX + dx;
                            const spY = currentY;
                            const spZ = originZ + dz;

                            try {
                                const currentBlock = this.Dimension.getBlock({ x: spX, y: spY, z: spZ });
                                const belowBlock = this.Dimension.getBlock({ x: spX, y: spY - 1, z: spZ });

                                if (!currentBlock || !belowBlock) continue;
                                if (belowBlock.typeId.includes("_sign")) break;
                                if (belowBlock.typeId.includes("vine")) break;
                                if (belowBlock.typeId.includes("_leaves")) break;

                                const canReplace = blockReplaceList.includes(currentBlock.typeId);
                                const isAirOrReplaceable = currentBlock.isAir || canReplace;

                                const isBelowBlockValid = !blockReplaceList.includes(belowBlock.typeId);

                                const isValidLocation = !belowBlock.isAir &&
                                    !belowBlock.isLiquid &&
                                    !currentBlock.isLiquid &&
                                    isAirOrReplaceable &&
                                    isBelowBlockValid;

                                if (isValidLocation) {
                                    validPositions.push({ x: spX, y: spY, z: spZ });
                                }
                            } catch (e) {
                                continue;
                            }
                        }
                    }
                }
                return validPositions;
            };

            const validPositions = generateCoordinatesWithinVolume();

            if (validPositions.length > 0) {
                const randomIndex = randomNumber(0, validPositions.length - 1);
                const selectedPosition = validPositions[randomIndex];

                this.Dimension.setBlockType(selectedPosition, blockID);

                system.run(() => {
                    try {
                        const blockInLocation = this.Dimension.getBlock(selectedPosition);
                        if (!blockInLocation || blockID !== blockInLocation.typeId) {
                            return;
                        }

                        const signComponent = blockInLocation.getComponent("minecraft:sign");
                        if (signComponent) {
                            try {
                                let permutation = blockInLocation.permutation;
                                if (permutation && blockID === "minecraft:standing_sign") {
                                    permutation = permutation.withState("ground_sign_direction", randomNumber(0, 15));
                                    blockInLocation.setPermutation(permutation);
                                }
                            } catch (permError) {
                                console.error(`Erro ao definir permutação: ${permError}`);
                            }

                            signComponent.setText({
                                rawtext: [
                                    { text: "§l§c" },
                                    { translate: `herobrine.sign.message_${randomNumber(1, 15)}` }
                                ]
                            });

                        }
                    } catch (componentError) {
                        console.error(`Erro no componente da placa em ${JSON.stringify(selectedPosition)}:`, componentError);
                    }
                });
            }

        } catch (error) {
            console.error("Erro em signSpawn:", error);
        }
    }

    herobrineChat() {
        try {
            world.sendMessage({
                rawtext: [
                    { text: `<` }, { translate: "entity.hrb:herobrine.name" }, { text: "> " },
                    { translate: `herobrine.herobrine_chat.message_${randomNumber(1, 10)}` }
                ]
            });
        } catch (e) {
            console.warn(`Failed to send Herobrine chat message: ${e}`);
        }
    }

    randomEvent() {
        try {
            const locationFix = {
                x: Math.round(this.herobrineData.location.x),
                y: Math.round(this.herobrineData.location.y),
                z: Math.round(this.herobrineData.location.z)
            };

            locationFix.y = Math.max(-64, Math.min(319, locationFix.y));

            const eventType = randomNumber(1, 4);

            switch (eventType) {
                case 1:
                    if (this.herobrineSettings.world_vandalism) {
                        this.Dimension.spawnEntity("minecraft:lightning_bolt", locationFix);
                    } else {
                        this.scarySound();
                    }
                    break;

                case 2:
                    if (this.herobrineSettings.world_vandalism) {
                        for (let i = 0; i < randomNumber(1, 5); i++) {
                            try { this.Dimension.spawnEntity("minecraft:silverfish", locationFix); } catch { }
                        }
                    } else {
                        this.scarySound();
                    }
                    break;

                case 3:
                    for (let i = 0; i < randomNumber(1, 5); i++) {
                        try {
                            const zombie = this.Dimension.spawnEntity("minecraft:zombie", locationFix);
                            // equipRandomItemsToZombie(zombie);
                        } catch { }
                    }
                    break;

                case 4:
                    if (this.herobrineSettings.world_vandalism) {
                        this.Dimension.createExplosion(locationFix, 3, { causesFire: true });
                    } else {
                        this.scarySound();
                    }
                    break;
            }
        } catch (error) {
            console.error("Error during Herobrine random event:", error);
        }
    }
}

const blockReplaceList = [
    "minecraft:snow_layer",
    "minecraft:leaf_litter",
    "minecraft:short_grass",
]